package LWP::Protocol::proxyproto;

use warnings;
use strict;

our $VERSION = '0.01'; # VERSION

require LWP::Protocol;
our @ISA = qw(LWP::Protocol);

our @supported_schemes = ( 'https', 'http' );

sub request {
    my($self, $request, $proxy, $arg, $size, $timeout) = @_;
    my $url = $request->uri;
    my $scheme = $url->scheme;

    if(!defined $proxy) {
	    return HTTP::Response->new( &HTTP::Status::RC_BAD_REQUEST,
		    'proxyproto schema can only be used with a proxy!');
    }

    if( ! grep { $scheme eq $_ } @supported_schemes ) {
	    return HTTP::Response->new( &HTTP::Status::RC_BAD_REQUEST,
		    $scheme.' protocol scheme is not supported by '. __PACKAGE__ );
    }

    my $connect_scheme = $scheme."::proxyproto";
    my $socket_scheme = $connect_scheme."::Socket";

    for my $scheme ( $connect_scheme, $socket_scheme ) {
        eval 'require LWP::Protocol::'.$scheme; ## no critic
      if( $@ ) {
          return HTTP::Response->new( &HTTP::Status::RC_BAD_REQUEST,
            'could not load '.$connect_scheme.' protocol support: '.$@);
      }
    }

    my $protocol = LWP::Protocol::create($connect_scheme, $self->{ua});
    $protocol->{proxy_connect_opts} = [
    	ProxyAddr => $proxy->host,
      ProxyPort => $proxy->port,
      ProxyUserinfo => $proxy->userinfo,
      Agent => $self->{ua},
    ];

    $protocol->request($request, undef, $arg, $size, $timeout);
}

1;

__END__

=head1 NAME

LWP::Protocol::proxyproto - Provides PROXY protocol v1 support for LWP::UserAgent

=head1 SYNOPSIS

  use LWP::UserAgent;

  $ua = LWP::UserAgent->new(); 
  $ua->proxy(['https', 'http'], 'proxyproto://server.example.com');

  $ua->credentials("proxyproto", "source", "fd00:c0ff:ee::bcc", "1234");
  $ua->credentials("proxyproto", "dest", "fd00:c0ff:ee::1", "443");

  $ua->get('https://www.example.com');

=head1 PROXY Protocol Configuration

PROXY protocol connection metadata is passed to the module by abusing the
credential functionality as this seems like the least worst way to politely
pass in custom data.

PROXY protocol requires a source address and port, and a destination address
and port. You can use IPv4 or IPv6 addresses - the module will work out the
right thing to do as long as the addresses are valid. Both addresses must be
the same IP version.

  $ua->credentials("proxyproto", "source", "fd00:c0ff:ee::bcc", "1234");
  $ua->credentials("proxyproto", "dest", "fd00:c0ff:ee::1", "443");

SSL certificate verification will continue to be based on the request 
parameters - the PROXY protocol configuration will have no effect.

=head1 DESCRIPTION

The LWP::Protocol::proxyproto module provides support for PROXY Protocol; a
convenient way to safely transport connection information such as a client's
address across multiple layers of NAT or TCP proxies.

See the protocol documentation for other implementation details:

  https://www.haproxy.org/download/1.8/doc/proxy-protocol.txt

This module currently only implements the human readable version 1 of the
protocol.

=head1 ACKNOWLEDGEMENTS

This module borrows heavily from LWP::Protocol::connect by Markus Benning.

=head1 AUTHOR

Ben Charlton <benc@cpan.org>

=head1 COPYRIGHT

Copyright 2019 Mythic Beasts

This library is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

